package com.smfreports.activityreport;
                                                               
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.*;
import java.util.stream.Collectors;

import org.apache.commons.cli.*;

import static java.util.Comparator.*;

import com.blackhillsoftware.smf.SmfRecord;
import com.blackhillsoftware.smf.SmfRecordReader;
import com.blackhillsoftware.smf.smf14.Smf14Record;
import com.blackhillsoftware.smf.smf17.Smf17Record;
import com.blackhillsoftware.smf.smf18.Smf18Record;
import com.blackhillsoftware.smf.smf61.Smf61Record;
import com.blackhillsoftware.smf.smf62.Smf62Record;
import com.blackhillsoftware.smf.smf64.Smf64Record;
import com.blackhillsoftware.smf.smf65.Smf65Record;
import com.blackhillsoftware.util.DD;
                               
import com.smfreports.common.*;

public class ActiveDatasets                                                                            
{   
    public static void main(String[] args) throws IOException, ParseException                                   
    {   
        CommandLine cmd = initOptions(args);
        
        MatchList ignoreJobs = readIgnoreList(cmd, "ignorejobs");          
        MatchList ignoreDatasets = readIgnoreList(cmd, "ignoredatasets");  

        Map<String, ActiveDatasetEntry> datasets = new HashMap<>();
        
        for (String inputFile : cmd.getArgList())
        {    
            System.out.format("Reading %s%n", inputFile);
            
            try (SmfRecordReader reader = SmfRecordReader.fromName(inputFile)
                    .include(14)
                    .include(15)
                    .include(17)
                    .include(18)
                    .include(61)
                    .include(62)
                    .include(64)
                    .include(65))
            { 
                for (SmfRecord record : reader) 
                {                    
                    switch (record.recordType())
                    {
                    // 14 and 15 use the same mapping
                    case 14: // Read
                    {
                        Smf14Record r14 = Smf14Record.from(record);
                        if (!r14.smf14tds()) // ignore temporary datasets 
                        {
                            String dataset = Gdg.maskIfGdg(r14.smfjfcb1().jfcbdsnm());
                            String jobname = r14.smf14jbn();
                            if (!ignoreJobs.matchValue(jobname)
                                    && !ignoreDatasets.matchValue(dataset))
                            {
                                datasets.computeIfAbsent(dataset, key -> new ActiveDatasetEntry(dataset))
                                    .addReader(jobname);
                            }
                        }
                        break;
                    }
                    case 15: // Update
                        {
                            Smf14Record r15 = Smf14Record.from(record);
                            if (!r15.smf14tds()) // ignore temporary datasets 
                            {
                                String dataset = Gdg.maskIfGdg(r15.smfjfcb1().jfcbdsnm());
                                String jobname = r15.smf14jbn();
                                if (!ignoreJobs.matchValue(jobname)
                                        && !ignoreDatasets.matchValue(dataset))
                                {
                                    datasets.computeIfAbsent(dataset, key -> new ActiveDatasetEntry(dataset))
                                        .addWriter(jobname);
                                }
                            }
                            break;
                        }
                    case 17: // scratch
                        {
                            Smf17Record r17 = Smf17Record.from(record);
                            String dataset = Gdg.maskIfGdg(r17.smf17dsn());
                            String jobname = r17.smf17jbn();
                            if (!ignoreJobs.matchValue(jobname)
                                    && !ignoreDatasets.matchValue(dataset))
                            {
                                datasets.computeIfAbsent(dataset, key -> new ActiveDatasetEntry(dataset))
                                    .addWriter(jobname);
                            }
                            break;
                        }
                    case 18: // rename
                        {
                            // Count rename as 2 events, scratch of old name and update of new name.
                            Smf18Record r18 = Smf18Record.from(record);
                            String dataset1 = Gdg.maskIfGdg(r18.smf18ods());
                            String dataset2 = Gdg.maskIfGdg(r18.smf18nds());
                            String jobname = r18.smf18jbn();
                            if (!ignoreJobs.matchValue(jobname)
                                    && !ignoreDatasets.matchValue(dataset1)
                                    && !ignoreDatasets.matchValue(dataset2))
                            {
                                datasets.computeIfAbsent(dataset1, key -> new ActiveDatasetEntry(dataset1))
                                    .addWriter(jobname);
                                datasets.computeIfAbsent(dataset2, key -> new ActiveDatasetEntry(dataset2))
                                	.addWriter(jobname);

                            }
                            break;
                        }
                    case 61: // ICF define
                        {
                            Smf61Record r61 = Smf61Record.from(record);
                            String dataset = Gdg.maskIfGdg(r61.smf61enm());
                            String jobname = r61.smf61jnm();
                            if (!ignoreJobs.matchValue(jobname)
                                    && !ignoreDatasets.matchValue(dataset))
                            {
                                datasets.computeIfAbsent(dataset, key -> new ActiveDatasetEntry(dataset))
                                    .addWriter(jobname);
                            }
                            break;
                        }
                    case 62: // VSAM open
                        {
                            Smf62Record r62 = Smf62Record.from(record);
                            String dataset = Gdg.maskIfGdg(r62.smf62dnm());
                            String jobname = r62.smf62jbn();
                            if (!ignoreJobs.matchValue(jobname)
                                    && !ignoreDatasets.matchValue(dataset))
                            {
                                datasets.computeIfAbsent(dataset, key -> new ActiveDatasetEntry(dataset))
                                    .addWriter(jobname);
                            }
                            break;
                        }
                    case 64: // VSAM Status
                        {
                            Smf64Record r64 = Smf64Record.from(record);
                            String dataset = Gdg.maskIfGdg(r64.smf64dnm());
                            String jobname = r64.smf64jbn();
                            if (!ignoreJobs.matchValue(jobname)
                                    && !ignoreDatasets.matchValue(dataset))
                            {
                                datasets.computeIfAbsent(dataset, key -> new ActiveDatasetEntry(dataset))
                                    .addWriter(jobname);
                            }
                            break;
                        }
                    case 65: // ICF delete
                        {
                            Smf65Record r65 = Smf65Record.from(record);
                            String dataset = Gdg.maskIfGdg(r65.smf65enm());
                            String jobname = r65.smf65jnm();
                            if (!ignoreJobs.matchValue(jobname)
                                    && !ignoreDatasets.matchValue(dataset))
                            {
                                datasets.computeIfAbsent(dataset, key -> new ActiveDatasetEntry(dataset))
                                    .addWriter(jobname);
                            }
                            break;
                        }
                    }    
                }
            }
        }
               
        if (cmd.hasOption("csv"))
        {
            writeCsvReport(datasets);
        }
        else
        {
        	writeReport(datasets);
        }           
        // List ignored jobs and datasets for information and validation
        System.out.format("%n%s%n", "Ignored Jobs:");
        ignoreJobs.getMatches().stream()
            .sorted()
            .forEachOrdered(job -> System.out.format("%s%n", job));
        
        System.out.format("%n%s%n", "Ignored Datasets:");
        ignoreDatasets.getMatches().stream()
            .sorted()
            .forEachOrdered(job -> System.out.format("%s%n", job));
        
    }

    private static void writeCsvReport(Map<String, ActiveDatasetEntry> datasetmap) 
    {

        System.out.format("%s,%s,%s%n", 
        		"Dataset", 
        		"Jobname",
        		"Access");

    	datasetmap.entrySet().stream()
            .sorted(Map.Entry.comparingByKey())
            .forEachOrdered(entry ->
	        {
	        	entry.getValue().getWriters().stream()
	        		.sorted()
	        		.forEachOrdered(writer ->
	        		{
	        	        System.out.format("\"%s\",\"%s\",\"%s\"%n",
	        	        		entry.getKey(), 
	        	        		writer,
	        	        		"Update");      			
	        		});
	        	
	        	entry.getValue().getReaders().stream()
	        		.sorted()
	        		.forEachOrdered(reader ->
	        		{
	        	        System.out.format("\"%s\",\"%s\",\"%s\"%n", 
	        	        		entry.getKey(), 
	        	        		reader,
	        	        		"Read");      			
	        		});
	        });
    }
    
    private static void writeReport(Map<String, ActiveDatasetEntry> datasetmap) 
    {
        List<ActiveDatasetEntry> datasets = datasetmap.values().stream()
                .sorted(comparing(ActiveDatasetEntry::getDatasetName))
                .collect(Collectors.toList());
        
        for (ActiveDatasetEntry entry : datasets)
        {
            List<String> readJobs = entry.getReaders().stream()
                    .sorted().collect(Collectors.toList());
            List<String> writerJobs = entry.getWriters().stream()
                    .sorted().collect(Collectors.toList());
            
            System.out.format("%n%-44s %-8s %-8s%n", entry.getDatasetName(), "Update", "Read");   // print dataset name

            int readcount = readJobs.size();
            int writecount = writerJobs.size();
            
            for (int i=0; i < readcount || i < writecount; i++)
            {
                System.out.format("%-44s %-8s %-8s%n",
                        "",
                        i < writecount ? writerJobs.get(i) : "",
                        i < readcount ? readJobs.get(i) : ""
                    );
            }
        }
    }
    
    private static MatchList readIgnoreList(CommandLine cmd, String optionName) throws IOException 
    {
        MatchList ignore = new MatchList();     
        String ignoreOption = cmd.getOptionValue(optionName); 
        if(ignoreOption != null)
        {
            ignore = ignoreOption.startsWith("//DD:") ?
                new MatchList(DD.readAllLines(ignoreOption)) :
                new MatchList(Files.readAllLines(new File(ignoreOption).toPath()));
        }
        return ignore;
    }
    
    private static void printUsage(Options options) 
    {
        HelpFormatter formatter = new HelpFormatter();
        formatter.printHelp( ActiveDatasets.class.getName() + " [options] <input> ...", 
                System.lineSeparator() +
                "List datasets appearing in SMF data", 
                options, 
                System.lineSeparator() + 
                "<input> : File(s) containing SMF records. Binary data, RECFM=U or V[B] including RDW." +
                System.lineSeparator() +
                System.lineSeparator() +
                "<ignorejoblist>, <ignoredatasetlist> and <input> can all be file names or DDNames in the format: //DD:DDNAME");
    }
    
    private static CommandLine initOptions(String[] args) throws ParseException 
    {
        Options options = new Options();
        options.addOption(
                Option.builder("h")
                    .longOpt("help")
                    .desc("print this message and exit")
                    .build());
        options.addOption("csv", "write output in CSV format");
        options.addOption( 
                Option.builder("ignorejobs")
                    .desc("a file or DD containing a list of jobs to exclude from the report" )
                    .hasArg()
                    .argName("ignorejoblist")
                    .build());
        options.addOption( 
                Option.builder("ignoredatasets")
                    .desc("a file or DD containing a list of datasets to exclude from the report" )
                    .hasArg()
                    .argName("ignoredatasetlist")
                    .build());
        
        CommandLineParser parser = new DefaultParser();
        CommandLine cmd = parser.parse(options, args);
        
        if (args.length == 0 || cmd.hasOption("help"))
        {
            printUsage(options);
            System.exit(0);
        }
        
        return cmd;
    }
    
}                                                                                               