package com.smfreports.activityreport;

import java.time.LocalDateTime;

import com.blackhillsoftware.smf.smf30.Smf30Record;

public class ActiveJobEntry 
{
    public ActiveJobEntry(String name)
    {
        jobName = name;
    }    
    
    public void add(Smf30Record r30)
    {
        if (r30.completionSection() != null)
        {
            count = getCount() + 1;
        }
        
        LocalDateTime submitTime = r30.identificationSection().smf30red()
                .atTime(r30.identificationSection().smf30ret());
        if (lastRunTime == null || submitTime.isAfter(getLastRunTime()))
        {
            lastRunTime = submitTime; 
        }
        
        if (r30.processorAccountingSection() != null)
        {
            double cpuTime = r30.processorAccountingSection().smf30cptSeconds()
                    + r30.processorAccountingSection().smf30cpsSeconds()
                    + r30.processorAccountingSection().smf30TimeOnZiipSeconds() * r30.performanceSection().smf30znf() / 256;
            
            if (cpuTime > getMaxCpu())
            {
                maxCpu = cpuTime;
                maxCpuSubmitTime = submitTime;
            }
        }
        
        if (r30.ioActivitySection() != null)
        {
            long excp = r30.ioActivitySection().smf30tex();
            if (excp > getMaxExcp())
            {
                maxExcp = excp;
                maxExcpSubmitTime = submitTime;
            }
        }
    }
    
    public String getJobName() 
    {
        return jobName;
    }

    public int getCount() {
        return count;
    }

    public LocalDateTime getMaxCpuSubmitTime() {
        return maxCpuSubmitTime;
    }

    public LocalDateTime getMaxExcpSubmitTime() {
        return maxExcpSubmitTime;
    }

    public LocalDateTime getLastRunTime() {
        return lastRunTime;
    }
    
    public double getMaxCpu() {
        return maxCpu;
    }

    public long getMaxExcp() {
        return maxExcp;
    }

    private String jobName;
    private int count = 0;
    
    private double maxCpu = 0;
    private LocalDateTime maxCpuSubmitTime;
    
    private long maxExcp = 0;
    private LocalDateTime maxExcpSubmitTime;

    private LocalDateTime lastRunTime;
    
    
}
